## 介绍
　　data-jdbc库基于SpringJDBC进行二次封装，拥有着强大性能的同时又不失简单、灵活。特性如下：
- 比SpringJDBC更方便好用、比SpringJPA更简单灵活
- **无侵入**：data-jdbc 在 SpringJDBC 的基础上进行扩展，只做增强不做改变，简化`CRUD`操作
- **依赖管理**：引入即可启动项目，关联druid实现SQL全监控
- **预防Sql注入**：内置Sql注入剥离器，有效预防Sql注入攻击
- **损耗小**：封装大量经过SQL优化处理的CRUD方法，直接面向对象操作，对比原生级CRUD处理，性能基本无损耗甚至更优
- **通用CRUD操作**：内置通用 DAO，通过继承方式即可实现单表大部分 CRUD 操作
- **更科学的分页**：分页参数自动解析，写分页等同于写基本List查询。更有优化型分页SQL检查
- **内置性能分析插件**：可输出Sql语句以及其执行时间，建议开发测试时启用该功能，能有效解决慢查询
- **类型强化**：支持原生级SQL查询，并强化原生查询结果，简单便捷 + 可维护组合（支持全JSON或全DO）
- **CRUD校验**：CRUD操作是否符合预期，更好的避免脏数据的产生与违规操作
- **全局异常处理**：CRUD操作相关异常统一处理，定位更精准，提示更友好，实现全局Restful风格

## 快速开始
### 引入模块
`yue-library-dependencies`作为父项目，在`pom.xml`文件中添加：
``` pom
<dependencies>
	<dependency>
		<groupId>ai.ylyue</groupId>
		<artifactId>yue-library-data-jdbc</artifactId>
	</dependency>
</dependencies>
```

### 配置数据源
`data-jdbc`就是SpringJDBC的封装，数据源配置如下：
```yaml
spring:
  datasource: 
    druid: 
      url: jdbc:mysql://localhost:3306/database?characterEncoding=utf-8&useSSL=false
      username: root
      password: 123456
```

### 简单使用
`data-jdbc`所有的CRUD方法都在`DB`类里面，所以使用时只需要直接注入即可，推荐采用继承`DBDAO 或 DBRepository`方式。<br>
<font color=red>注意：sql数据表中主键的DDL最好同下面一样。</font>
```ddl
`id` bigint(20) unsigned NOT NULL AUTO_INCREMENT COMMENT '表自增ID'
```
主键ID：bigint类型、无符号、自动递增、不能为NULL
> 其实这样做也符合了“阿里巴巴Java开发手册”MySQL 数据库-建表规约第九条：<br>
> 9. 【强制】表必备三字段：id, create_time, update_time。
说明：其中 id 必为主键，类型为 bigint unsigned、单表时自增、步长为 1。create_time, update_time
的类型均为 datetime 类型。

**DBDAO：**
```java
@Repository
public class DataJdbcExampleDAO extends DBDAO {

	@Override
	protected String tableName() {
		return "tableName";
	}
	
}
```

**DBRepository：**
```java
@Repository
public class DataJdbcExampleTDAO extends DBRepository<UserDO> {

	@Override
	protected String tableName() {
		return "user";
	}
	
}
```

### <font color=red>DAO内置实现：</font>
`DBDAO`为 JSON 对象提供服务

`DBRepository`为 DO 对象提供服务，字段映射支持下划线与驼峰自动识别转换

实际中可能会遇到类型转换问题，可使用 `Convert` 类进行转换，支持DO、Json、List等相互转换

```java
// 插入数据
insert(JSONObject)
// 插入数据-批量
insertBatch(JSONObject[])
// 删除
delete(Long)
// 删除-安全的
deleteSafe(Long)
// 更新-ById
updateById(JSONObject)
// 单个
get(Long)
// 列表-全部
listAll()
// 分页
page(PageIPO)
// 分页-降序
pageDESC(PageIPO)
```

## 其他
### 绝对条件查询参数whereSql化
```java
paramToWhereSql(JSONObject)
```

### 预期值判断
```java
// 是否有数据
isDataSize(long)

// ------ 预期值判断 ------
// 更新所影响的行数
int updateRowsNumber = 1;
// 预期值
int expectedValue = 1;

// 有返回值-布尔
// 判断更新所影响的行数是否 <b>等于</b> 预期值
db.isUpdateAndExpectedEqual(updateRowsNumber, expectedValue);
// 判断更新所影响的行数是否 <b>大于等于</b> 预期值
db.isUpdateAndExpectedGreaterThanEqual(updateRowsNumber, expectedValue);

// 无返回值-DBException异常
// 判断更新所影响的行数是否 <b>等于</b> 预期值
db.updateAndExpectedEqual(updateRowsNumber, expectedValue);
// 判断更新所影响的行数是否 <b>大于等于</b> 预期值
db.updateAndExpectedGreaterThanEqual(updateRowsNumber, expectedValue);
// 确认批量更新每组参数所影响的行数，是否 <b>全部都等于</b> 同一个预期值
int[] updateRowsNumberArray = {1, 1};
db.updateBatchAndExpectedEqual(updateRowsNumberArray, expectedValue);
```

### 查询结果转换
由于原生`SpringJDBC`查询单条数据，若为空将抛出异常。所以一般会采用查询多条数据方式，然后再转换成单条数据对象：
```java
resultToJson(List<JSONObject>)
resultToObject(List<T>)
```