# 接口质检标准
yue-library已提供 [Restful](https://ylyue.cn/#/base/Restful) 支持

## 为什么要有接口质检标准，接口质检标准是什么？
　　接口质检标准是检验API是否规范的一种标准，可帮助我们统一接口管理。一个比较好的实践方案，应该做到语义简洁明了，看到 URL 就知道是做什么的，就如同 RESTful 中所提到的那样。

## 接口文档规范
- <font color=red>（必须）</font>做到业务逻辑自测
- <font color=red>（必须）</font>提供标准的请求示例与响应示例
- <font color=red>（必须）</font>请求示例配备字段说明，并做好字段规则排序
- <font color=red>（重要）</font>接口字段命名与数据库保持一致
- <font color=red>（重要）</font>响应示例：复杂业务接口提供详细的字段注释
- （推荐）响应示例：日常业务接口提供简单注释

## 接口命名规范
　　URI命名风格采用增强 Restful API 接口形式，接口命名需遵守API接口版本控制规则，并且以URL前缀风格做一定的安全认证区分。
- GET 查询接口（用于查询数据）
- POST 提交接口（用于插入数据、提交安全加密数据等）
- PUT 更新接口（用于更新数据，具有破坏性）
- DELETE 删除接口（用于删除数据，具有破坏性）

### 接口命名前缀
|前缀	|说明								|示例									|
|--		|--									|--										|
|open	|公开接口，未登录					|/open/v1.2/user/resetPassword			|
|auth	|认证接口，需要登录					|/auth/v1.2/user/updatePassword			|
|private|私有接口，仅内部调用，服务间调用	|/private/v1.2/user/loadUserByUsername	|

### API版本控制
yue-library已提供 [Restful](https://ylyue.cn/#/base/Restful) 支持

API的版本号统一放入URL，如：https://api.example.com/v{n}/

- **v{n}**：n代表版本号，分为整形和浮点型
- **整形版本号**：大功能版本发布形式；具有当前版本状态下的所有API接口，例如：v1,v2
- **浮点型**：为小版本号，只具备补充api的功能，其他api都默认调用大版本号的API，例如v1.1,v1.2

### RESTful URL路径参数
**前端传参：**以apizza示例，apizza支持RESTful URL，如果在URL填写:key，然后在Query参数中填写对应 key，发送请求的时候插件将会替换URL中的 :key

![前端传参](接口质检标准_files/1.png)

**后端取参：**
```java
@PostMapping("/{opt}")
public Result<?> pathVariable(@PathVariable String opt) {
	return ResultInfo.success(opt);
}
```

### GET规范
- /user/get 用户-单个（获得单条数据）
- /user/list 用户-列表（获得列表数据）
- /user/page 用户-分页（获得分页数据）
- /user/isUser 用户-确认（获得Boolean值）

### POST规范
- /user/insert 用户-插入（插入数据）
- /wxPay/app 微信支付-APP（业务操作或涉及加密等）
- /aliPay/app 支付宝支付-APP（业务操作或涉及加密等）
- /open/login 登录（业务操作或涉及加密等）
- /open/logout 登出（业务操作或涉及加密等）

### PUT规范
- /user/updateById 用户-更新-ById（更新单条数据）
- /user/updateNickname 用户-更新-昵称（更新具体值）
- /user/resetPassword 用户-重置密码（更新业务）

### DELETE规范
- /user/delete 用户-删除（删除单条数据）
- /user/deleteByNickname 用户-删除-ByNickname（删除符合条件数据）

## API接口规范
### 协议
- API与用户的通信协议，总是使用HTTPS协议，确保交互数据的传输安全。
- 应该尽量将API部署在专用域名之下

### 安全
为了保证接口接收到的数据不是被篡改以及防止信息泄露造成损失，对敏感数据进行加密及签名，并且以 [接口命名前缀](#接口命名前缀) 做一定的安全认证区分。

**数据加密**：api接口请求参数一律采用RSA进行加解密，在客户端使用公钥对请求参数进行加密，在服务端使用对数私钥据进行解密，防止信息泄露。<br>
**签名**：为了防止请求数据在网络传输过程中被恶意篡改，对所有非查询接口增加数字签名，签名原串为对请求参数进行自然排序，通过私钥加签后放入sign参数中。<br>
**时间戳**：api接口中增加时间戳timestamp字段，作用：固定时间范围内，减少同一请求被暴力调用的次数。

### 请求定义
**常见请求header约定**

|参数名称		|参数类型	|最大长度	|描述									|示例									|
|--				|--			|--			|--										|--										|
|tenant_id		|bigint		|20			|租户ID，用于系统多租户区分				|1										|
|client_id		|String		|20			|客户端请求来源APP WAP PC				|APP									|
|access_token	|String		|36			|访问令牌，UUID5						|65dbec7a-1df5-5413-bf41-9d4e41ee4ba7	|
|timestamp		|String		|17			|发送请求的时间,格式:yyyyMMddHHmmssSSS	|20180505121212222						|

**常见请求参数约定**

|参数名称	|参数类型	|最大长度	|描述							|示例				|
|--			|--			|--			|--								|--					|
|page		|Int		|5			|当前页							|1					|
|limit		|Int		|6			|每页显示条数					|20					|
|sign_type	|String		|10			|生成签名字符串所使用的算法类型	|RSA				|
|sign		|String		|344		|请求参数签名串					|djdu7dusufiusgfu	|
|sortFields	|String		|50			|查询排序字段					|id,name,age		|
|sortTypes	|String		|50			|查询排序类型					|ASC,DESC,ASC		|

### 响应定义
**响应体格式约定**

|参数名称	|参数类型	|最大长度	|描述			|示例																	|
|--			|--			|--			|--				|--																		|
|code		|Int		|3			|请求状态码		|200																	|
|msg		|String		|30			|请求提示信息	|成功																	|
|flag		|Boolean	|			|请求状态		|true																	|
|count		|Int		|			|分页统计条数	|null																	|
|data		|Object		|			|响应数据		|【钉钉】通知结果：{\"errcode\":0,\"success\":true,\"errmsg\":\"ok\"}	|

示例：
```json
{
    "code": 200,
    "msg": "成功",
    "flag": true,
    "count": null,
    "data": "【钉钉】通知结果：{\"errcode\":0,\"success\":true,\"errmsg\":\"ok\"}"
}
```

**code定义：**
```java
public enum ResultEnum {
	
	// 200 - 正确结果
	SUCCESS(200, "成功"),
	LOGGED_IN(210, "会话未注销，无需登录"),
	
    // 300 - 资源、重定向、定位等提示
	RESOURCE_ALREADY_INVALID(300, "资源已失效"),
	FILE_EMPTY(310, "文件上传请求错误，获得文件信息为空，求先使用测试工具（如：postman）校验，同时文件必须有明确的匹配类型（如文本类型：.txt）"),
    
    // 400 - 客户端错误
	BAD_REQUEST(400, "错误的请求，参数校验未通过，请参照API核对后重试"),
	UNAUTHORIZED(401, "未登录或登录已失效（Unauthorized）"),
	ATTACK(402, "非法访问"),
	FORBIDDEN(403, "无权限（Forbidden）"),
	NOT_FOUND(404, "Not Found"),
	METHOD_NOT_ALLOWED(405, "方法不允许（Method Not Allowed）"),
	GONE(410, "当前API接口版本已弃用，请客户端更新接口调用方式"),
	UNSUPPORTED_MEDIA_TYPE(415, "Unsupported Media Type"),
	TOO_MANY_REQUESTS(429, "频繁请求限流，请稍后重试（Too Many Requests）"),
	PARAM_VOID(432, "参数为空"),
	PARAM_CHECK_NOT_PASS(433, "参数校验未通过，请参照API核对后重试"),
	PARAM_VALUE_INVALID(434, "参数校验未通过，无效的value"),
	PARAM_DECRYPT_ERROR(435, "参数解密错误"),
	
    // 500 - 服务器错误
	INTERNAL_SERVER_ERROR(500, "服务器内部错误（Internal Server Error）"),
	ERROR(501, "请求错误"),
	BAD_GATEWAY(502, "Bad Gateway"),
	SERVICE_UNAVAILABLE(503, "Service Unavailable"),
	GATEWAY_TIMEOUT(504, "Gateway Timeout"),
	DATA_STRUCTURE(505, "数据结构异常"),
	DB_ERROR(506, "数据结构异常，请检查相应数据结构一致性"),
    CLIENT_FALLBACK(507, "哎哟喂！网络开小差了，请稍后重试..."),
    CLIENT_FALLBACK_ERROR(508, "哎哟喂！服务都被您挤爆了..."),
    TYPE_CONVERT_ERROR(509, "类型转换错误"),
    
	// 600 - 自定义错误提示
    DEV_CUSTOM_TYPE_PROMPT(600, "开发者自定义类型提示，请覆盖。");
	
}
```

### 非RESTful API需求
由于实际业务开展过程中，可能会出现各种的api不是简单的restful规范能实现的，因此需要一些api突破restful规范原则。

#### 页面级API**
把当前页面中需要用到的所有数据通过一个接口一次性返回全部数据。

例子：`api/v1/get-home-data` 返回首页用到的所有数据，但此类API存在缺陷，只要业务需求变动，该api就需要跟着变更。

#### 自定义组合API
把当前用户需要在第一时间内容加载的多个接口合并成一个请求发送到服务端，服务端根据请求内容，一次性把所有数据合并返回，相比于页面级API，具备更高的灵活性，同时又能很容易实现页面级API功能。

地址:api/v1/testApi

传入参数：
```json
{
    "data": [
        {
            "url": "api1",
            "type": "get",
            "data": {}
        },
        {
            "url": "api2",
            "type": "get",
            "data": {}
        },
        {
            "url": "api3",
            "type": "get",
            "data": {}
        }
    ]
}
```

返回数据：
```json
{
  "code": 200,
  "flag": true,
  "msg": "成功",
  "data": [
      {"code": 200,"msg": "","data": []},
      {"code": 200,"msg": "","data": []},
      {"code": 200,"msg": "","data": []}
  ]
}
```

### 接口协作管理工具
使用 [apizza](https://apizza.net/pro/#/) 代替postman、传统API接口文档编写、接口流程测试、Mock服务